==> cactoos_0.48.orig.tar.gz: cactoos-0.48/src/main/java/org/cactoos/scalar/Solid.java <==
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2020 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.cactoos.scalar;

import org.cactoos.Scalar;

/**
 * Cached and synchronized version of a Scalar.
 *
 * <p>Objects of this class are thread safe.
 *
 * @param <T> Type of result
 * @see Sticky
 * @see Synced
 * @since 0.24
 */
public final class Solid<T> implements Scalar<T> {

    /**
     * Origin.
     */
    private final Scalar<T> origin;

    /**
     * Cache.
     */
    private volatile T cache;

    /**
     * Sync lock.
     */
    private final Object lock;

    /**
     * Ctor.
     * @param origin The Scalar to cache and sync
     */
    public Solid(final Scalar<T> origin) {
        this.origin = origin;
        this.lock = new Object();
    }

    @Override
    public T value() throws Exception {
        if (this.cache == null) {
            synchronized (this.lock) {
                if (this.cache == null) {
                    this.cache = this.origin.value();
                }
            }
        }
        return this.cache;
    }
}

==> cactoos_0.54.0.orig.tar.gz: cactoos-0.54.0/src/main/java/org/cactoos/scalar/Solid.java <==
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2017-2022 Yegor Bugayenko
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.cactoos.scalar;

import org.cactoos.Scalar;

/**
 * Cached and synchronized version of a Scalar.
 *
 * <p>Objects of this class are thread safe.
 *
 * @param <T> Type of result
 * @see Sticky
 * @see Synced
 * @since 0.24
 */
public final class Solid<T> implements Scalar<T> {

    /**
     * Origin.
     */
    private final Scalar<? extends T> origin;

    /**
     * Cache.
     */
    private volatile T cache;

    /**
     * Sync lock.
     */
    private final Object lock;

    /**
     * Ctor.
     * @param origin The Scalar to cache and sync
     */
    public Solid(final Scalar<? extends T> origin) {
        this.origin = origin;
        this.lock = new Object();
    }

    @Override
    public T value() throws Exception {
        if (this.cache == null) {
            synchronized (this.lock) {
                if (this.cache == null) {
                    this.cache = this.origin.value();
                }
            }
        }
        return this.cache;
    }
}
